from __future__ import annotations
from typing import TYPE_CHECKING, Any, Callable, Generic, Iterable, Iterator, TypeVar, overload

T = TypeVar('T')

if TYPE_CHECKING:
    from Live.Base import IntU64Vector, IntVector, Vector
    from Live.ClipSlot import ClipSlot
    from Live.DeviceParameter import DeviceParameter
    from Live.Envelope import Envelope
    from Live.Groove import Groove
    from Live.LomObject import LomObject
    from Live.Track import Track



class Clip(LomObject):
    """
    This class represents a Clip in Live. It can be either an Audio
    Clip or a MIDI Clip, in an Arrangement or the Session, depending
    on the Track (Slot) it lives in.
    """

    class View(LomObject):
        """Representing the view aspects of a Clip."""

        @property
        def _live_ptr(self) -> int:
            ...

        @property
        def canonical_parent(self) -> Clip:
            """Get the canonical parent of the clip view."""
            ...

        @property
        def grid_is_triplet(self) -> bool:
            """Get/set wether the grid is showing in triplet mode."""
            ...

        @grid_is_triplet.setter
        def grid_is_triplet(self, value: bool) -> None: ...

        @property
        def grid_quantization(self) -> GridQuantization:
            """Get/set clip grid quantization resolution."""
            ...

        @grid_quantization.setter
        def grid_quantization(self, value: GridQuantization) -> None: ...

        def hide_envelope(self) -> None:
            """Hide the envelope view."""
            ...

        def select_envelope_parameter(self, device_parameter: DeviceParameter | None, /) -> None:
            """Select the given device parameter in the envelope view."""
            ...

        def show_envelope(self) -> None:
            """Show the envelope view."""
            ...

        def show_loop(self) -> None:
            """Show the entire loop in the detail view."""
            ...

    @property
    def _live_ptr(self) -> int:
        ...

    def add_color_index_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "color_index" has changed.
        """
        ...

    def add_color_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "color" has changed.
        """
        ...

    def add_end_marker_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "end_marker" has changed.
        """
        ...

    def add_end_time_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "end_time" has changed.
        """
        ...

    def add_file_path_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "file_path" has changed.
        """
        ...

    def add_gain_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "gain" has changed.
        """
        ...

    def add_groove_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "groove" has changed.
        """
        ...

    def add_has_envelopes_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "has_envelopes" has changed.
        """
        ...

    def add_is_overdubbing_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "is_overdubbing" has changed.
        """
        ...

    def add_is_recording_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "is_recording" has changed.
        """
        ...

    def add_launch_mode_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "launch_mode" has changed.
        """
        ...

    def add_launch_quantization_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "launch_quantization" has changed.
        """
        ...

    def add_legato_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "legato" has changed.
        """
        ...

    def add_loop_end_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "loop_end" has changed.
        """
        ...

    def add_loop_jump_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "loop_jump" has changed.
        """
        ...

    def add_loop_start_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "loop_start" has changed.
        """
        ...

    def add_looping_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "looping" has changed.
        """
        ...

    def add_muted_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "muted" has changed.
        """
        ...

    def add_name_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "name" has changed.
        """
        ...

    def add_new_notes(self, notes: Iterable[MidiNoteSpecification] | None, /) -> IntU64Vector:
        """
        Expects a Python iterable holding a number of Live.Clip.MidiNoteSpecification
        objects. The objects will be used to construct new notes in the clip.
        """
        ...

    def add_notes_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "notes" has changed.
        """
        ...

    def add_pitch_coarse_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "pitch_coarse" has changed.
        """
        ...

    def add_pitch_fine_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "pitch_fine" has changed.
        """
        ...

    def add_playing_position_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "playing_position" has changed.
        """
        ...

    def add_playing_status_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "playing_status" has changed.
        """
        ...

    def add_position_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "position" has changed.
        """
        ...

    def add_ram_mode_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "ram_mode" has changed.
        """
        ...

    def add_signature_denominator_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "signature_denominator" has changed.
        """
        ...

    def add_signature_numerator_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "signature_numerator" has changed.
        """
        ...

    def add_start_marker_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "start_marker" has changed.
        """
        ...

    def add_start_time_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "start_time" has changed.
        """
        ...

    def add_velocity_amount_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "velocity_amount" has changed.
        """
        ...

    def add_warp_marker(self, warp_marker: WarpMarker | None, /) -> None:
        """
        Available for AudioClips only.
        Adds the specified warp marker, if possible.
        """
        ...

    def add_warp_markers_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "warp_markers" has changed.
        """
        ...

    def add_warp_mode_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "warp_mode" has changed.
        """
        ...

    def add_warping_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "warping" has changed.
        """
        ...

    def apply_note_modifications(self, notes: MidiNoteVector | None, /) -> None:
        """
        Expects a list of notes as returned from get_notes_extended. The content
        of the list will be used to modify existing notes in the clip, based on
        matching note IDs.
        This function should be used when modifying existing notes, e.g. changing the
        velocity or start time. The function ensures that per-note events attached to
        the modified notes are preserved. This is NOT the case when replacing notes
        via a combination of remove_notes_extended and add_new_notes.
        The given list can be a subset of the notes in the clip, but it must not
        contain any notes that are not present in the clip.
        """
        ...

    def automation_envelope(self, device_parameter: DeviceParameter | None, /) -> Envelope:
        """Return the envelope for the given parameter.Returns None if the envelope doesn't exist.Returns None for Arrangement clips.Returns None for parameters from a different track."""
        ...

    @property
    def automation_envelopes(self) -> Vector[Envelope]:
        """Const access to a list of all automation envelopes for this clip."""
        ...

    @property
    def available_warp_modes(self) -> IntVector:
        """
        Available for AudioClips only.
        Get/Set the available warp modes, that can be used.
        """
        ...

    def beat_to_sample_time(self, beat_time: float | None, /) -> float:
        """
        Available for AudioClips only.
        Converts the given beat time to sample time. Raises an error if the sample is not warped.
        """
        ...

    @property
    def canonical_parent(self) -> ClipSlot:
        """Get the canonical parent of the Clip."""
        ...

    def clear_all_envelopes(self) -> None:
        """Clears all envelopes for this clip."""
        ...

    def clear_envelope(self, device_parameter: DeviceParameter | None, /) -> None:
        """Clears the envelope of this clips given parameter."""
        ...

    @property
    def color(self) -> int:
        """Get/set access to the color of the Clip (RGB)."""
        ...

    @color.setter
    def color(self, value: int) -> None: ...

    def color_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "color".
        """
        ...

    @property
    def color_index(self) -> int:
        """Get/set access to the color index of the Clip."""
        ...

    @color_index.setter
    def color_index(self, value: int) -> None: ...

    def color_index_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "color_index".
        """
        ...

    def create_automation_envelope(self, device_parameter: DeviceParameter | None, /) -> Envelope:
        """Creates an envelope for a given parameter and returns it.This should only be used if the envelope doesn't exist.Raises an error if the envelope can't be created."""
        ...

    def crop(self) -> None:
        """
        Crops the clip. The region that is cropped depends on whether the clip is
        looped or not. If looped, the region outside of the loop is removed.
        If not looped, the region outside the start and end markers is removed.
        """
        ...

    def deselect_all_notes(self) -> None:
        """De-selects all notes present in the clip."""
        ...

    def duplicate_loop(self) -> None:
        """
        Make the loop two times longer and duplicates notes and envelopes.
        Duplicates the clip start/end range if the clip is not looped.
        """
        ...

    def duplicate_notes_by_id(self, note_ids: Iterable[int] | None, destination_time: float | None = None, transposition_amount: int = 0, /) -> IntU64Vector:
        """
        Duplicate all notes matching the given note IDs.
        If the optional destination_time is not provided, new notes will be inserted
        after the last selected note. This behavior can be observed when duplicating
        notes in the Live GUI.
        If the transposition_amount is specified, the notes in the region will be
        transposed by the number of semitones.
        Raises an error on audio clips.
        """
        ...

    def duplicate_region(self, region_start: float | None, region_length: float | None, destination_time: float | None, pitch: int = -1, transposition_amount: int = 0, /) -> None:
        """
        Duplicate the notes in the specified region to the destination_time.
        Only notes of the specified pitch are duplicated or all if pitch is -1.
        If the transposition_amount is not 0, the notes in the region will
        be transposed by the transpose_amount of semitones.Raises an error on audio clips.
        """
        ...

    @property
    def end_marker(self) -> float:
        """Get/Set the Clips end marker pos in beats/seconds (unit depends on warping)."""
        ...

    @end_marker.setter
    def end_marker(self, value: float) -> None: ...

    def end_marker_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "end_marker".
        """
        ...

    @property
    def end_time(self) -> float:
        """Get the clip's end time."""
        ...

    def end_time_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "end_time".
        """
        ...

    @property
    def file_path(self) -> str:
        """Get the path of the file represented by the Audio Clip."""
        ...

    def file_path_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "file_path".
        """
        ...

    def fire(self) -> None:
        """(Re)Start playing this Clip."""
        ...

    @property
    def gain(self) -> float:
        """
        Available for AudioClips only.
        Read/write access to the gain setting of the
        Audio Clip
        """
        ...

    @gain.setter
    def gain(self, value: float) -> None: ...

    @property
    def gain_display_string(self) -> str:
        """Return a string with the gain as dB value"""
        ...

    def gain_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "gain".
        """
        ...

    def get_all_notes_extended(self) -> MidiNoteVector:
        """
        Returns a list of all MIDI notes from the clip, regardless of their position
        relative to the start and end markers/loop start and loop end.
        Each note is represented by a Live.Clip.MidiNote object.
        The returned list can be modified freely, but modifications will not
        be reflected in the MIDI clip until apply_note_modifications is called.
        """
        ...

    def get_notes(self, from_time: float | None, from_pitch: int | None, time_span: float | None, pitch_span: int | None, /) -> tuple[tuple[int, float, float, float, bool], ...]:
        """
        Returns a tuple of tuples where each inner tuple represents
        a note starting in the given pitch- and time range.
        The inner tuple contains pitch, time, duration, velocity, and mute state.
        """
        ...

    def get_notes_by_id(self, note_ids: Iterable[int] | None, /) -> MidiNoteVector:
        """Return a list of MIDI notes matching the given note IDs."""
        ...

    def get_notes_extended(self, from_pitch: int | None, pitch_span: int | None, from_time: float | None, time_span: float | None, /) -> MidiNoteVector:
        """
        Returns a list of MIDI notes from the given pitch and time range.
        Each note is represented by a Live.Clip.MidiNote object.
        The returned list can be modified freely, but modifications will not
        be reflected in the MIDI clip until apply_note_modifications is called.
        """
        ...

    def get_selected_notes(self) -> tuple[tuple[int, float, float, float, bool], ...]:
        """
        Returns a tuple of tuples where each inner tuple
        represents a selected note. The inner tuple contains
        pitch, time, duration, velocity, and mute state.
        """
        ...

    def get_selected_notes_extended(self) -> MidiNoteVector:
        """
        Returns a list of all MIDI notes from the clip that are currently selected.
        Each note is represented by a Live.Clip.MidiNote object.
        The returned list can be modified freely, but modifications will not
        be reflected in the MIDI clip until apply_note_modifications is called.
        """
        ...

    @property
    def groove(self) -> Groove | None:
        """Get the groove associated with this clip."""
        ...

    @groove.setter
    def groove(self, value: Groove | None) -> None: ...

    def groove_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "groove".
        """
        ...

    @property
    def has_envelopes(self) -> bool:
        """Will notify if the clip gets his first envelope or the last envelope is removed."""
        ...

    def has_envelopes_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "has_envelopes".
        """
        ...

    @property
    def has_groove(self) -> bool:
        """Returns true if a groove is associated with this clip."""
        ...

    @property
    def is_arrangement_clip(self) -> bool:
        """
        return true if this Clip is an Arrangement Clip.
        A Clip can be either a Session or Arrangement Clip.
        """
        ...

    @property
    def is_audio_clip(self) -> bool:
        """
        Return true if this Clip is an Audio Clip.
        A Clip can be either an Audioclip or a MIDI Clip.
        """
        ...

    @property
    def is_midi_clip(self) -> bool:
        """
        return true if this Clip is a MIDI Clip.
        A Clip can be either an Audioclip or a MIDI Clip.
        """
        ...

    @property
    def is_overdubbing(self) -> bool:
        """returns true if the Clip is recording overdubs"""
        ...

    def is_overdubbing_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "is_overdubbing".
        """
        ...

    @property
    def is_playing(self) -> bool:
        """
        Get/Set if this Clip is currently playing. If the Clips trigger mode
        is set to a quantization value, the Clip will not start playing immediately.
        If you need to know wether the Clip was triggered, use the is_triggered property.
        """
        ...

    @is_playing.setter
    def is_playing(self, value: bool) -> None: ...

    @property
    def is_recording(self) -> bool:
        """returns true if the Clip was triggered to record or is recording."""
        ...

    def is_recording_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "is_recording".
        """
        ...

    @property
    def is_session_clip(self) -> bool:
        """
        return true if this Clip is a Session Clip.
        A Clip can be either a Session or Arrangement Clip.
        """
        ...

    @property
    def is_take_lane_clip(self) -> bool:
        """
        return true if this Clip is a Take Lane Clip.
        A Take Lane Clip is also always an Arrangement Clip.
        """
        ...

    @property
    def is_triggered(self) -> bool:
        """returns true if the Clip was triggered or is playing."""
        ...

    @property
    def launch_mode(self) -> int:
        """Get/Set access to the launch mode setting of the Clip."""
        ...

    @launch_mode.setter
    def launch_mode(self, value: int) -> None: ...

    def launch_mode_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "launch_mode".
        """
        ...

    @property
    def launch_quantization(self) -> int:
        """Get/Set access to the launch quantization setting of the Clip."""
        ...

    @launch_quantization.setter
    def launch_quantization(self, value: int) -> None: ...

    def launch_quantization_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "launch_quantization".
        """
        ...

    @property
    def legato(self) -> bool:
        """Get/Set access to the legato setting of the Clip"""
        ...

    @legato.setter
    def legato(self, value: bool) -> None: ...

    def legato_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "legato".
        """
        ...

    @property
    def length(self) -> float:
        """Get to the Clips length in beats/seconds (unit depends on warping)."""
        ...

    @property
    def loop_end(self) -> float:
        """Get/Set the loop end pos of this Clip in beats/seconds (unit depends on warping)."""
        ...

    @loop_end.setter
    def loop_end(self, value: float) -> None: ...

    def loop_end_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "loop_end".
        """
        ...

    def loop_jump_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "loop_jump".
        """
        ...

    @property
    def loop_start(self) -> float:
        """Get/Set the Clips loopstart pos in beats/seconds (unit depends on warping)."""
        ...

    @loop_start.setter
    def loop_start(self, value: float) -> None: ...

    def loop_start_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "loop_start".
        """
        ...

    @property
    def looping(self) -> bool:
        """
        Get/Set the Clips 'loop is enabled' flag
        .Only Warped Audio Clips or MIDI Clip can be looped.
        """
        ...

    @looping.setter
    def looping(self, value: bool) -> None: ...

    def looping_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "looping".
        """
        ...

    def move_playing_pos(self, beats: float | None, /) -> None:
        """
        Jump forward or backward by the specified relative amount in beats.
        Will do nothing, if the Clip is not playing.
        """
        ...

    def move_warp_marker(self, marker_beat_time: float | None, beat_time_distance: float | None, /) -> None:
        """
        Available for AudioClips only.
        Moves the specified warp marker by the specified beat time amount, if possible.
        """
        ...

    @property
    def muted(self) -> bool:
        """Read/write access to the mute state of the Clip."""
        ...

    @muted.setter
    def muted(self, value: bool) -> None: ...

    def muted_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "muted".
        """
        ...

    @property
    def name(self) -> str:
        """Read/write access to the name of the Clip."""
        ...

    @name.setter
    def name(self, value: str) -> None: ...

    def name_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "name".
        """
        ...

    def note_number_to_name(self, midi_pitch: int | None, /) -> str:
        """
        Return a human-readable name for the given MIDI note number.
        Takes into account the scale and tonal spelling settings of the clip,
        as well as the current tuning system (if any)
        """
        ...

    def notes_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "notes".
        """
        ...

    @property
    def pitch_coarse(self) -> int:
        """
        Available for AudioClips only.
        Read/write access to the pitch (in halftones) setting of the
        Audio Clip, ranging from -48 to 48
        """
        ...

    @pitch_coarse.setter
    def pitch_coarse(self, value: int) -> None: ...

    def pitch_coarse_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "pitch_coarse".
        """
        ...

    @property
    def pitch_fine(self) -> float:
        """
        Available for AudioClips only.
        Read/write access to the pitch fine setting of the
        Audio Clip, ranging from -500 to 500
        """
        ...

    @pitch_fine.setter
    def pitch_fine(self, value: float) -> None: ...

    def pitch_fine_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "pitch_fine".
        """
        ...

    @property
    def playing_position(self) -> float:
        """
        Constant access to the current playing position of the clip.
        The returned value is the position in beats for midi and warped audio clips,
        or in seconds for unwarped audio clips. Stopped clips will return 0.
        """
        ...

    def playing_position_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "playing_position".
        """
        ...

    def playing_status_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "playing_status".
        """
        ...

    @property
    def position(self) -> float:
        """Get/Set the loop position of this Clip in beats/seconds (unit depends on warping)."""
        ...

    @position.setter
    def position(self, value: float) -> None: ...

    def position_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "position".
        """
        ...

    def quantize(self, quantization_grid: int | None, amount: float | None, /) -> None:
        """Quantize all notes in a clip or align warp markers."""
        ...

    def quantize_pitch(self, note: int | None, source: int | None, amount: float | None, /) -> None:
        """Quantize all the notes of a given pitch. Raises an error on audio clips."""
        ...

    @property
    def ram_mode(self) -> bool:
        """
        Available for AudioClips only.
        Read/write access to the Ram mode setting of the Audio Clip
        """
        ...

    @ram_mode.setter
    def ram_mode(self, value: bool) -> None: ...

    def ram_mode_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "ram_mode".
        """
        ...

    def remove_color_index_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "color_index".
        """
        ...

    def remove_color_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "color".
        """
        ...

    def remove_end_marker_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "end_marker".
        """
        ...

    def remove_end_time_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "end_time".
        """
        ...

    def remove_file_path_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "file_path".
        """
        ...

    def remove_gain_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "gain".
        """
        ...

    def remove_groove_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "groove".
        """
        ...

    def remove_has_envelopes_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "has_envelopes".
        """
        ...

    def remove_is_overdubbing_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "is_overdubbing".
        """
        ...

    def remove_is_recording_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "is_recording".
        """
        ...

    def remove_launch_mode_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "launch_mode".
        """
        ...

    def remove_launch_quantization_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "launch_quantization".
        """
        ...

    def remove_legato_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "legato".
        """
        ...

    def remove_loop_end_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "loop_end".
        """
        ...

    def remove_loop_jump_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "loop_jump".
        """
        ...

    def remove_loop_start_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "loop_start".
        """
        ...

    def remove_looping_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "looping".
        """
        ...

    def remove_muted_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "muted".
        """
        ...

    def remove_name_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "name".
        """
        ...

    def remove_notes(self, from_time: float | None, from_pitch: int | None, time_span: float | None, pitch_span: int | None, /) -> None:
        """Delete all notes starting in the given pitch- and time range."""
        ...

    def remove_notes_by_id(self, note_ids: Iterable[int] | None, /) -> None:
        """
        Delete all notes matching the given note IDs.
        This function should NOT be used to implement modification of existing notes
        (i.e. in combination with add_new_notes), as that leads to loss of per-note
        events. apply_note_modifications must be used instead for modifying existing
        notes.
        """
        ...

    def remove_notes_extended(self, from_pitch: int | None, pitch_span: int | None, from_time: float | None, time_span: float | None, /) -> None:
        """
        Delete all notes starting in the given pitch and time range.
        This function should NOT be used to implement modification of existing notes
        (i.e. in combination with add_new_notes), as that leads to loss of per-note
        events. apply_note_modifications must be used instead for modifying existing
        notes.
        """
        ...

    def remove_notes_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "notes".
        """
        ...

    def remove_pitch_coarse_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "pitch_coarse".
        """
        ...

    def remove_pitch_fine_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "pitch_fine".
        """
        ...

    def remove_playing_position_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "playing_position".
        """
        ...

    def remove_playing_status_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "playing_status".
        """
        ...

    def remove_position_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "position".
        """
        ...

    def remove_ram_mode_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "ram_mode".
        """
        ...

    def remove_signature_denominator_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "signature_denominator".
        """
        ...

    def remove_signature_numerator_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "signature_numerator".
        """
        ...

    def remove_start_marker_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "start_marker".
        """
        ...

    def remove_start_time_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "start_time".
        """
        ...

    def remove_velocity_amount_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "velocity_amount".
        """
        ...

    def remove_warp_marker(self, beat_time: float | None, /) -> None:
        """
        Available for AudioClips only.
        Removes the specified warp marker, if possible.
        """
        ...

    def remove_warp_markers_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "warp_markers".
        """
        ...

    def remove_warp_mode_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "warp_mode".
        """
        ...

    def remove_warping_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "warping".
        """
        ...

    def replace_selected_notes(self, notes: tuple[tuple[int, float, float, float, bool], ...] | None, /) -> None:
        """
        Called with a tuple of tuples where each inner tuple represents
        a note in the same format as returned by get_selected_notes. The
        notes described that way will then be used to replace the old selection.
        """
        ...

    @property
    def sample_length(self) -> int:
        """
        Available for AudioClips only.
        Get the sample length in sample time or -1 if there is no sample available.
        """
        ...

    @property
    def sample_rate(self) -> float:
        """
        Available for AudioClips only.
        Read-only access to the Clip's sampling rate.
        """
        ...

    def sample_to_beat_time(self, sample_time: float | None, /) -> float:
        """
        Available for AudioClips only.
        Converts the given sample time to beat time. Raises an error if the sample is not warped.
        """
        ...

    def scrub(self, scrub_position: float | None, /) -> None:
        """
        Scrubs inside a clip.
        scrub_position defines the position in beats that the scrub will start from.
        The scrub will continue until stop_scrub is called.
        Global quantization applies to the scrub's position and length.
        """
        ...

    def seconds_to_sample_time(self, seconds: float | None, /) -> float:
        """
        Available for AudioClips only.
        Converts the given seconds to sample time. Raises an error if the sample is warped.
        """
        ...

    def select_all_notes(self) -> None:
        """Selects all notes present in the clip."""
        ...

    def select_notes_by_id(self, note_ids: Iterable[int] | None, /) -> None:
        """Selects all notes matching the given note IDs."""
        ...

    def set_fire_button_state(self, state: bool | None, /) -> None:
        """Set the clip's fire button state directly. Supports all launch modes."""
        ...

    def set_notes(self, notes: tuple[tuple[int, float, float, float, bool], ...] | None, /) -> None:
        """
        Called with a tuple of tuples where each inner tuple represents
        a note in the same format as returned by get_notes. The
        notes described that way will then be added to the clip.
        """
        ...

    @property
    def signature_denominator(self) -> int:
        """Get/Set access to the global signature denominator of the Clip."""
        ...

    @signature_denominator.setter
    def signature_denominator(self, value: int) -> None: ...

    def signature_denominator_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "signature_denominator".
        """
        ...

    @property
    def signature_numerator(self) -> int:
        """Get/Set access to the global signature numerator of the Clip."""
        ...

    @signature_numerator.setter
    def signature_numerator(self, value: int) -> None: ...

    def signature_numerator_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "signature_numerator".
        """
        ...

    @property
    def start_marker(self) -> float:
        """Get/Set the Clips start marker pos in beats/seconds (unit depends on warping)."""
        ...

    @start_marker.setter
    def start_marker(self, value: float) -> None: ...

    def start_marker_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "start_marker".
        """
        ...

    @property
    def start_time(self) -> float:
        """Get the clip's start time offset. For Session View clips, this is the time the clip was started. For Arrangement View clips, this is the offset within the arrangement."""
        ...

    def start_time_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "start_time".
        """
        ...

    def stop(self) -> None:
        """Stop playing this Clip."""
        ...

    def stop_scrub(self) -> None:
        """Stops the current scrub."""
        ...

    @property
    def velocity_amount(self) -> float:
        """Get/Set access to the velocity to volume amount of the Clip."""
        ...

    @velocity_amount.setter
    def velocity_amount(self, value: float) -> None: ...

    def velocity_amount_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "velocity_amount".
        """
        ...

    @property
    def view(self) -> View:
        """Get the view of the Clip."""
        ...

    @property
    def warp_markers(self) -> WarpMarkerVector:
        """
        Available for AudioClips only.
        Get the warp markers for this audio clip.
        """
        ...

    def warp_markers_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "warp_markers".
        """
        ...

    @property
    def warp_mode(self) -> int:
        """
        Available for AudioClips only.
        Get/Set the warp mode for this audio clip.
        """
        ...

    @warp_mode.setter
    def warp_mode(self, value: int) -> None: ...

    def warp_mode_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "warp_mode".
        """
        ...

    @property
    def warping(self) -> bool:
        """
        Available for AudioClips only.
        Get/Set if this Clip is timestreched.
        """
        ...

    @warping.setter
    def warping(self, value: bool) -> None: ...

    def warping_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "warping".
        """
        ...

    @property
    def will_record_on_start(self) -> bool:
        """returns true if the Clip will record on being started."""
        ...

class ClipLaunchQuantization(int):
    q_global: int = 0
    q_none: int = 1
    q_8_bars: int = 2
    q_4_bars: int = 3
    q_2_bars: int = 4
    q_bar: int = 5
    q_half: int = 6
    q_half_triplet: int = 7
    q_quarter: int = 8
    q_quarter_triplet: int = 9
    q_eighth: int = 10
    q_eighth_triplet: int = 11
    q_sixteenth: int = 12
    q_sixteenth_triplet: int = 13
    q_thirtysecond: int = 14

class GridQuantization(int):
    no_grid: int = 0
    g_8_bars: int = 1
    g_4_bars: int = 2
    g_2_bars: int = 3
    g_bar: int = 4
    g_half: int = 5
    g_quarter: int = 6
    g_eighth: int = 7
    g_sixteenth: int = 8
    g_thirtysecond: int = 9
    count: int = 10

class LaunchMode(int):
    trigger: int = 0
    gate: int = 1
    toggle: int = 2
    repeat: int = 3

class MidiNote:
    """An object representing a MIDI Note"""

    @property
    def duration(self) -> float:
        ...

    @duration.setter
    def duration(self, value: float) -> None: ...

    @property
    def mute(self) -> bool:
        ...

    @mute.setter
    def mute(self, value: bool) -> None: ...

    @property
    def note_id(self) -> int:
        """
        A numerical ID that's unique within the originating clip of the note. Not to be
        used directly, but important for other API calls, namely apply_note_modifications.
        """
        ...

    @property
    def pitch(self) -> int:
        ...

    @pitch.setter
    def pitch(self, value: int) -> None: ...

    @property
    def probability(self) -> float:
        ...

    @probability.setter
    def probability(self, value: float) -> None: ...

    @property
    def release_velocity(self) -> float:
        ...

    @release_velocity.setter
    def release_velocity(self, value: float) -> None: ...

    @property
    def start_time(self) -> float:
        ...

    @start_time.setter
    def start_time(self, value: float) -> None: ...

    @property
    def velocity(self) -> float:
        ...

    @velocity.setter
    def velocity(self, value: float) -> None: ...

    @property
    def velocity_deviation(self) -> float:
        ...

    @velocity_deviation.setter
    def velocity_deviation(self, value: float) -> None: ...

class MidiNoteSpecification:
    """
    An object specifying the data for creating a MIDI note. To be used with the
    add_new_notes function.
    """

    def __init__(self, pitch: int, start_time: float, duration: float, velocity: float = 100.0, mute: bool = False, probability: float = 1.0, velocity_deviation: float = 0.0, release_velocity: float = 64.0) -> None: ...

class MidiNoteVector(Vector[MidiNote]):
    """A container for holding MIDI notes from Live."""

    def append(self, value: MidiNote | None, /) -> None:
        ...

    def extend(self, values: Iterable[MidiNote] | None, /) -> None:
        ...

class WarpMarker:
    """This class represents a WarpMarker type."""

    def __init__(self, sample_time: float, beat_time: float) -> None: ...

    @property
    def beat_time(self) -> float:
        """A WarpMarker's beat time."""
        ...

    @property
    def sample_time(self) -> float:
        """A WarpMarker's sample time."""
        ...

class WarpMarkerVector(Vector[WarpMarker]):
    """A container for returning warp markers from Live."""

    def append(self, value: WarpMarker | None, /) -> None:
        ...

    def extend(self, values: Iterable[WarpMarker] | None, /) -> None:
        ...

class WarpMode(int):
    beats: int = 0
    tones: int = 1
    texture: int = 2
    repitch: int = 3
    complex: int = 4
    rex: int = 5
    complex_pro: int = 6
    count: int = 7

__all__ = ['Clip', 'ClipLaunchQuantization', 'GridQuantization', 'LaunchMode', 'MidiNote', 'MidiNoteSpecification', 'MidiNoteVector', 'WarpMarker', 'WarpMarkerVector', 'WarpMode']



from __future__ import annotations
from typing import TYPE_CHECKING, Any, Callable, Iterable

if TYPE_CHECKING:
    from Live.Base import Vector
    from Live.Device import ATimeableValueVector, Device, DeviceType
    from Live.DeviceIO import DeviceIO
    from Live.DeviceParameter import DeviceParameter
    from Live.Track import Track



class MaxDevice(Device):
    """This class represents a Max for Live device."""

    @property
    def _live_ptr(self) -> int:
        ...

    def add_audio_inputs_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "audio_inputs" has changed.
        """
        ...

    def add_audio_outputs_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "audio_outputs" has changed.
        """
        ...

    def add_bank_parameters_changed_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "bank_parameters_changed" has changed.
        """
        ...

    def add_midi_inputs_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "midi_inputs" has changed.
        """
        ...

    def add_midi_outputs_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "midi_outputs" has changed.
        """
        ...

    @property
    def audio_inputs(self) -> Vector[DeviceIO]:
        """Const access to a list of all audio inputs of the device."""
        ...

    def audio_inputs_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "audio_inputs".
        """
        ...

    @property
    def audio_outputs(self) -> Vector[DeviceIO]:
        """Const access to a list of all audio outputs of the device."""
        ...

    def audio_outputs_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "audio_outputs".
        """
        ...

    def bank_parameters_changed_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "bank_parameters_changed".
        """
        ...

    @property
    def can_compare_ab(self) -> bool:
        """Returns true if the Device has the capability to AB compare."""
        ...

    @property
    def can_have_chains(self) -> bool:
        """Returns true if the device is a rack."""
        ...

    @property
    def can_have_drum_pads(self) -> bool:
        """Returns true if the device is a drum rack."""
        ...

    @property
    def canonical_parent(self) -> Track:
        """Get the canonical parent of the Device."""
        ...

    @property
    def class_display_name(self) -> str:
        """Return const access to the name of the device's class name as displayed in Live's browser and device chain"""
        ...

    @property
    def class_name(self) -> str:
        """Return const access to the name of the device's class."""
        ...

    def get_bank_count(self) -> int:
        """Get the number of parameter banks. This is related to hardware control surfaces."""
        ...

    def get_bank_name(self, bank_index: int | None, /) -> str:
        """Get the name of a parameter bank given by index. This is related to hardware control surfaces."""
        ...

    def get_bank_parameters(self, bank_index: int | None, /) -> list[int]:
        """Get the indices of parameters of the given bank index. Empty slots are marked as -1. Bank index -1 refers to the best-of bank. This function is related to hardware control surfaces."""
        ...

    def get_value_item_icons(self, device_parameter: DeviceParameter | None, /) -> list[str]:
        """Get a list of icon identifier strings for a list parameter's values.An empty string is given where no icon should be displayed.An empty list is given when no icons should be displayed.This is related to hardware control surfaces."""
        ...

    @property
    def is_active(self) -> bool:
        """Return const access to whether this device is active. This will be false bothwhen the device is off and when it's inside a rack device which is off."""
        ...

    @property
    def is_using_compare_preset_b(self) -> bool:
        """Returns whether the Device has loaded the preset in compare slot B. Only relevant if can_compare_ab, otherwise errors."""
        ...

    @is_using_compare_preset_b.setter
    def is_using_compare_preset_b(self, value: bool) -> None: ...

    @property
    def latency_in_ms(self) -> float:
        """Returns the latency of the device in ms."""
        ...

    @property
    def latency_in_samples(self) -> int:
        """Returns the latency of the device in samples."""
        ...

    @property
    def midi_inputs(self) -> Vector:
        """Const access to a list of all midi outputs of the device."""
        ...

    def midi_inputs_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "midi_inputs".
        """
        ...

    @property
    def midi_outputs(self) -> Vector:
        """Const access to a list of all midi outputs of the device."""
        ...

    def midi_outputs_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "midi_outputs".
        """
        ...

    @property
    def name(self) -> str:
        """Return access to the name of the device."""
        ...

    @name.setter
    def name(self, value: str) -> None: ...

    @property
    def parameters(self) -> ATimeableValueVector:
        """Const access to the list of available automatable parameters for this device."""
        ...

    def remove_audio_inputs_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "audio_inputs".
        """
        ...

    def remove_audio_outputs_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "audio_outputs".
        """
        ...

    def remove_bank_parameters_changed_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "bank_parameters_changed".
        """
        ...

    def remove_midi_inputs_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "midi_inputs".
        """
        ...

    def remove_midi_outputs_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "midi_outputs".
        """
        ...

    @property
    def type(self) -> DeviceType:
        """Return the type of the device."""
        ...

    @property
    def view(self) -> Device.View:
        """Representing the view aspects of a device."""
        ...

__all__ = ['MaxDevice']



from __future__ import annotations
from typing import TYPE_CHECKING, Any, Callable, Iterable

if TYPE_CHECKING:
    from Live.Base import Vector
    from Live.Groove import Groove
    from Live.LomObject import LomObject
    from Live.Song import Song



class GroovePool(LomObject):
    """This class represents the groove pool in Live."""

    @property
    def _live_ptr(self) -> int:
        ...

    def add_grooves_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "grooves" has changed.
        """
        ...

    @property
    def canonical_parent(self) -> Song:
        """Get the canonical parent of the groove pool."""
        ...

    @property
    def grooves(self) -> Vector[Groove]:
        """Access to the list of grooves"""
        ...

    def grooves_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "grooves".
        """
        ...

    def remove_grooves_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "grooves".
        """
        ...

__all__ = ['GroovePool']



from __future__ import annotations
from typing import TYPE_CHECKING, Any, Callable, Iterable

if TYPE_CHECKING:
    from Live.Base import Vector
    from Live.ChainMixerDevice import ChainMixerDevice
    from Live.Device import Device
    from Live.LomObject import LomObject
    from Live.RackDevice import RackDevice
    from Live.Track import DeviceContainer, Track



class Chain(DeviceContainer):
    """This class represents a group device chain in Live."""

    @property
    def _live_ptr(self) -> int:
        ...

    def add_color_index_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "color_index" has changed.
        """
        ...

    def add_color_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "color" has changed.
        """
        ...

    def add_devices_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "devices" has changed.
        """
        ...

    def add_is_auto_colored_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "is_auto_colored" has changed.
        """
        ...

    def add_mute_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "mute" has changed.
        """
        ...

    def add_muted_via_solo_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "muted_via_solo" has changed.
        """
        ...

    def add_name_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "name" has changed.
        """
        ...

    def add_solo_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "solo" has changed.
        """
        ...

    @property
    def canonical_parent(self) -> RackDevice:
        """Get the canonical parent of the chain."""
        ...

    @property
    def color(self) -> int:
        """Access the color index of the Chain."""
        ...

    @color.setter
    def color(self, value: int) -> None: ...

    def color_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "color".
        """
        ...

    @property
    def color_index(self) -> int:
        """Access the color index of the Chain."""
        ...

    @color_index.setter
    def color_index(self, value: int) -> None: ...

    def color_index_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "color_index".
        """
        ...

    def delete_device(self, device: int | None, /) -> None:
        """Remove a device identified by its index from the chain. Throws runtime error if bad index."""
        ...

    @property
    def devices(self) -> Vector[LomObject]:
        """Return const access to all available Devices that are present in the chains"""
        ...

    def devices_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "devices".
        """
        ...

    def duplicate_device(self, index: int | None, /) -> None:
        """Duplicate the device at the given index in the chain."""
        ...

    @property
    def has_audio_input(self) -> bool:
        """
        return True, if this Chain can be feed with an Audio signal. This is
        true for all Audio Chains.
        """
        ...

    @property
    def has_audio_output(self) -> bool:
        """
        return True, if this Chain sends out an Audio signal. This is
        true for all Audio Chains, and MIDI chains with an Instrument.
        """
        ...

    @property
    def has_midi_input(self) -> bool:
        """
        return True, if this Chain can be feed with an Audio signal. This is
        true for all MIDI Chains.
        """
        ...

    @property
    def has_midi_output(self) -> bool:
        """
        return True, if this Chain sends out MIDI events. This is
        true for all MIDI Chains with no Instruments.
        """
        ...

    def insert_device(self, device_name: str | None, device_index: int = -1, /) -> Device:
        """Add a device at a given index in the chain. At end if -1."""
        ...

    @property
    def is_auto_colored(self) -> bool:
        """
        Get/set access to the auto color flag of the Chain.
        If True, the Chain will always have the same color as the containing
        Track or Chain.
        """
        ...

    @is_auto_colored.setter
    def is_auto_colored(self, value: bool) -> None: ...

    def is_auto_colored_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "is_auto_colored".
        """
        ...

    @property
    def mixer_device(self) -> ChainMixerDevice:
        """
        Return access to the mixer device that holds the chain's mixer parameters:
        the Volume, Pan, and Sendamounts.
        """
        ...

    @property
    def mute(self) -> bool:
        """Mute/unmute the chain."""
        ...

    @mute.setter
    def mute(self, value: bool) -> None: ...

    def mute_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "mute".
        """
        ...

    @property
    def muted_via_solo(self) -> bool:
        """
        Return const access to whether this chain is muted due to some other chain
        being soloed.
        """
        ...

    def muted_via_solo_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "muted_via_solo".
        """
        ...

    @property
    def name(self) -> str:
        """Read/write access to the name of the Chain, as visible in the track header."""
        ...

    @name.setter
    def name(self, value: str) -> None: ...

    def name_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "name".
        """
        ...

    def remove_color_index_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "color_index".
        """
        ...

    def remove_color_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "color".
        """
        ...

    def remove_devices_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "devices".
        """
        ...

    def remove_is_auto_colored_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "is_auto_colored".
        """
        ...

    def remove_mute_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "mute".
        """
        ...

    def remove_muted_via_solo_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "muted_via_solo".
        """
        ...

    def remove_name_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "name".
        """
        ...

    def remove_solo_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "solo".
        """
        ...

    @property
    def solo(self) -> bool:
        """
        Get/Set the solo status of the chain. Note that this will not disable the
        solo state of any other Chain in the same rack. If you want exclusive solo,
        you have to disable the solo state of the other Chains manually.
        """
        ...

    @solo.setter
    def solo(self, value: bool) -> None: ...

    def solo_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "solo".
        """
        ...

__all__ = ['Chain']



from __future__ import annotations
from typing import TYPE_CHECKING, Any, Callable, Iterable

if TYPE_CHECKING:
    from Live.Base import StringVector
    from Live.Device import ATimeableValueVector, Device, DeviceType
    from Live.DeviceParameter import DeviceParameter
    from Live.Track import Track



class WavetableDevice(Device):
    """This class represents a Wavetable device."""

    @property
    def _live_ptr(self) -> int:
        ...

    def add_filter_routing_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "filter_routing" has changed.
        """
        ...

    def add_modulation_matrix_changed_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "modulation_matrix_changed" has changed.
        """
        ...

    def add_mono_poly_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "mono_poly" has changed.
        """
        ...

    def add_oscillator_1_effect_mode_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "oscillator_1_effect_mode" has changed.
        """
        ...

    def add_oscillator_1_wavetable_category_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "oscillator_1_wavetable_category" has changed.
        """
        ...

    def add_oscillator_1_wavetable_index_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "oscillator_1_wavetable_index" has changed.
        """
        ...

    def add_oscillator_1_wavetables_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "oscillator_1_wavetables" has changed.
        """
        ...

    def add_oscillator_2_effect_mode_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "oscillator_2_effect_mode" has changed.
        """
        ...

    def add_oscillator_2_wavetable_category_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "oscillator_2_wavetable_category" has changed.
        """
        ...

    def add_oscillator_2_wavetable_index_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "oscillator_2_wavetable_index" has changed.
        """
        ...

    def add_oscillator_2_wavetables_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "oscillator_2_wavetables" has changed.
        """
        ...

    def add_parameter_to_modulation_matrix(self, parameter: DeviceParameter | None, /) -> int:
        """Add a non-pitch parameter to the modulation matrix."""
        ...

    def add_poly_voices_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "poly_voices" has changed.
        """
        ...

    def add_unison_mode_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "unison_mode" has changed.
        """
        ...

    def add_unison_voice_count_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "unison_voice_count" has changed.
        """
        ...

    def add_visible_modulation_target_names_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "visible_modulation_target_names" has changed.
        """
        ...

    @property
    def can_compare_ab(self) -> bool:
        """Returns true if the Device has the capability to AB compare."""
        ...

    @property
    def can_have_chains(self) -> bool:
        """Returns true if the device is a rack."""
        ...

    @property
    def can_have_drum_pads(self) -> bool:
        """Returns true if the device is a drum rack."""
        ...

    @property
    def canonical_parent(self) -> Track:
        """Get the canonical parent of the Device."""
        ...

    @property
    def class_display_name(self) -> str:
        """Return const access to the name of the device's class name as displayed in Live's browser and device chain"""
        ...

    @property
    def class_name(self) -> str:
        """Return const access to the name of the device's class."""
        ...

    @property
    def filter_routing(self) -> int:
        """Return the current filter routing."""
        ...

    @filter_routing.setter
    def filter_routing(self, value: int) -> None: ...

    def filter_routing_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "filter_routing".
        """
        ...

    def get_modulation_target_parameter_name(self, target_index: int | None, /) -> str:
        """Get the parameter name of the modulation target at the given index."""
        ...

    def get_modulation_value(self, target_index: int | None, source: int | None, /) -> float:
        """Get the value of a modulation amount for the given target-source connection."""
        ...

    @property
    def is_active(self) -> bool:
        """Return const access to whether this device is active. This will be false bothwhen the device is off and when it's inside a rack device which is off."""
        ...

    def is_parameter_modulatable(self, parameter: DeviceParameter | None, /) -> bool:
        """Indicate whether the parameter is modulatable. Note that pitch parameters only exist in python and must be handled there."""
        ...

    @property
    def is_using_compare_preset_b(self) -> bool:
        """Returns whether the Device has loaded the preset in compare slot B. Only relevant if can_compare_ab, otherwise errors."""
        ...

    @is_using_compare_preset_b.setter
    def is_using_compare_preset_b(self, value: bool) -> None: ...

    @property
    def latency_in_ms(self) -> float:
        """Returns the latency of the device in ms."""
        ...

    @property
    def latency_in_samples(self) -> int:
        """Returns the latency of the device in samples."""
        ...

    def modulation_matrix_changed_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "modulation_matrix_changed".
        """
        ...

    @property
    def mono_poly(self) -> int:
        """Return the current voicing mode."""
        ...

    @mono_poly.setter
    def mono_poly(self, value: int) -> None: ...

    def mono_poly_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "mono_poly".
        """
        ...

    @property
    def name(self) -> str:
        """Return access to the name of the device."""
        ...

    @name.setter
    def name(self, value: str) -> None: ...

    @property
    def oscillator_1_effect_mode(self) -> int:
        """Return the current effect mode of the oscillator 1."""
        ...

    @oscillator_1_effect_mode.setter
    def oscillator_1_effect_mode(self, value: int) -> None: ...

    def oscillator_1_effect_mode_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "oscillator_1_effect_mode".
        """
        ...

    @property
    def oscillator_1_wavetable_category(self) -> int:
        """Return the current wavetable category of the oscillator 1."""
        ...

    @oscillator_1_wavetable_category.setter
    def oscillator_1_wavetable_category(self, value: int) -> None: ...

    def oscillator_1_wavetable_category_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "oscillator_1_wavetable_category".
        """
        ...

    @property
    def oscillator_1_wavetable_index(self) -> int:
        """Return the current wavetable index of the oscillator 1."""
        ...

    @oscillator_1_wavetable_index.setter
    def oscillator_1_wavetable_index(self, value: int) -> None: ...

    def oscillator_1_wavetable_index_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "oscillator_1_wavetable_index".
        """
        ...

    @property
    def oscillator_1_wavetables(self) -> StringVector:
        """Get a vector of oscillator 1's wavetable names."""
        ...

    def oscillator_1_wavetables_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "oscillator_1_wavetables".
        """
        ...

    @property
    def oscillator_2_effect_mode(self) -> int:
        """Return the current effect mode of the oscillator 2."""
        ...

    @oscillator_2_effect_mode.setter
    def oscillator_2_effect_mode(self, value: int) -> None: ...

    def oscillator_2_effect_mode_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "oscillator_2_effect_mode".
        """
        ...

    @property
    def oscillator_2_wavetable_category(self) -> int:
        """Return the current wavetable category of the oscillator 2."""
        ...

    @oscillator_2_wavetable_category.setter
    def oscillator_2_wavetable_category(self, value: int) -> None: ...

    def oscillator_2_wavetable_category_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "oscillator_2_wavetable_category".
        """
        ...

    @property
    def oscillator_2_wavetable_index(self) -> int:
        """Return the current wavetable index of the oscillator 2."""
        ...

    @oscillator_2_wavetable_index.setter
    def oscillator_2_wavetable_index(self, value: int) -> None: ...

    def oscillator_2_wavetable_index_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "oscillator_2_wavetable_index".
        """
        ...

    @property
    def oscillator_2_wavetables(self) -> StringVector:
        """Get a vector of oscillator 2's wavetable names."""
        ...

    def oscillator_2_wavetables_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "oscillator_2_wavetables".
        """
        ...

    @property
    def oscillator_wavetable_categories(self) -> StringVector:
        """Get a vector of the available wavetable categories."""
        ...

    @property
    def parameters(self) -> ATimeableValueVector:
        """Const access to the list of available automatable parameters for this device."""
        ...

    @property
    def poly_voices(self) -> int:
        """Return the current number of polyphonic voices. Uses the VoiceCount enumeration."""
        ...

    @poly_voices.setter
    def poly_voices(self, value: int) -> None: ...

    def poly_voices_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "poly_voices".
        """
        ...

    def remove_filter_routing_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "filter_routing".
        """
        ...

    def remove_modulation_matrix_changed_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "modulation_matrix_changed".
        """
        ...

    def remove_mono_poly_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "mono_poly".
        """
        ...

    def remove_oscillator_1_effect_mode_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "oscillator_1_effect_mode".
        """
        ...

    def remove_oscillator_1_wavetable_category_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "oscillator_1_wavetable_category".
        """
        ...

    def remove_oscillator_1_wavetable_index_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "oscillator_1_wavetable_index".
        """
        ...

    def remove_oscillator_1_wavetables_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "oscillator_1_wavetables".
        """
        ...

    def remove_oscillator_2_effect_mode_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "oscillator_2_effect_mode".
        """
        ...

    def remove_oscillator_2_wavetable_category_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "oscillator_2_wavetable_category".
        """
        ...

    def remove_oscillator_2_wavetable_index_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "oscillator_2_wavetable_index".
        """
        ...

    def remove_oscillator_2_wavetables_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "oscillator_2_wavetables".
        """
        ...

    def remove_poly_voices_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "poly_voices".
        """
        ...

    def remove_unison_mode_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "unison_mode".
        """
        ...

    def remove_unison_voice_count_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "unison_voice_count".
        """
        ...

    def remove_visible_modulation_target_names_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "visible_modulation_target_names".
        """
        ...

    def set_modulation_value(self, target_index: int | None, source: int | None, value: float | None, /) -> None:
        """Set the value of a modulation amount for the given target-source connection."""
        ...

    @property
    def type(self) -> DeviceType:
        """Return the type of the device."""
        ...

    @property
    def unison_mode(self) -> int:
        """Return the current unison mode."""
        ...

    @unison_mode.setter
    def unison_mode(self, value: int) -> None: ...

    def unison_mode_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "unison_mode".
        """
        ...

    @property
    def unison_voice_count(self) -> int:
        """Return the current number of unison voices."""
        ...

    @unison_voice_count.setter
    def unison_voice_count(self, value: int) -> None: ...

    def unison_voice_count_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "unison_voice_count".
        """
        ...

    @property
    def view(self) -> Device.View:
        """Representing the view aspects of a device."""
        ...

    @property
    def visible_modulation_target_names(self) -> StringVector:
        """Get the names of all the visible modulation targets."""
        ...

    def visible_modulation_target_names_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "visible_modulation_target_names".
        """
        ...

class EffectMode(int):
    none: int = 0
    frequency_modulation: int = 1
    sync_and_pulse_width: int = 2
    warp_and_fold: int = 3

class FilterRouting(int):
    serial: int = 0
    parallel: int = 1
    split: int = 2

class ModulationSource(int):
    amp_envelope: int = 0
    envelope_2: int = 1
    envelope_3: int = 2
    lfo_1: int = 3
    lfo_2: int = 4
    midi_velocity: int = 5
    midi_note: int = 6
    midi_pitch_bend: int = 7
    midi_channel_pressure: int = 8
    midi_mod_wheel: int = 9
    midi_random: int = 10

class UnisonMode(int):
    none: int = 0
    classic: int = 1
    slow_shimmer: int = 2
    fast_shimmer: int = 3
    phase_sync: int = 4
    position_spread: int = 5
    random_note: int = 6

class VoiceCount(int):
    two: int = 0
    three: int = 1
    four: int = 2
    five: int = 3
    six: int = 4
    seven: int = 5
    eight: int = 6

class Voicing(int):
    mono: int = 0
    poly: int = 1

__all__ = ['WavetableDevice', 'EffectMode', 'FilterRouting', 'ModulationSource', 'UnisonMode', 'VoiceCount', 'Voicing']



from __future__ import annotations
from typing import TYPE_CHECKING, Any, Callable, Iterable

if TYPE_CHECKING:
    from Live.Base import IntVector, Vector
    from Live.Chain import Chain
    from Live.Clip import Clip
    from Live.ClipSlot import ClipSlot
    from Live.Device import Device
    from Live.DeviceParameter import DeviceParameter
    from Live.Envelope import Envelope
    from Live.GroovePool import GroovePool
    from Live.LomObject import LomObject
    from Live.Scene import Scene
    from Live.Track import Track
    from Live.TuningSystem import TuningSystem



class Song(LomObject):
    """This class represents a Live set."""

    class View(LomObject):
        """Representing the view aspects of a Live document: The Session and Arrangerview."""

        @property
        def _live_ptr(self) -> int:
            ...

        def add_detail_clip_listener(self, callback: Callable | None, /) -> None:
            """
            Add a listener function or method, which will be called as soon as the
            property "detail_clip" has changed.
            """
            ...

        def add_draw_mode_listener(self, callback: Callable | None, /) -> None:
            """
            Add a listener function or method, which will be called as soon as the
            property "draw_mode" has changed.
            """
            ...

        def add_follow_song_listener(self, callback: Callable | None, /) -> None:
            """
            Add a listener function or method, which will be called as soon as the
            property "follow_song" has changed.
            """
            ...

        def add_selected_chain_listener(self, callback: Callable | None, /) -> None:
            """
            Add a listener function or method, which will be called as soon as the
            property "selected_chain" has changed.
            """
            ...

        def add_selected_parameter_listener(self, callback: Callable | None, /) -> None:
            """
            Add a listener function or method, which will be called as soon as the
            property "selected_parameter" has changed.
            """
            ...

        def add_selected_scene_listener(self, callback: Callable | None, /) -> None:
            """
            Add a listener function or method, which will be called as soon as the
            property "selected_scene" has changed.
            """
            ...

        def add_selected_track_listener(self, callback: Callable | None, /) -> None:
            """
            Add a listener function or method, which will be called as soon as the
            property "selected_track" has changed.
            """
            ...

        @property
        def canonical_parent(self) -> Song:
            """Get the canonical parent of the song view."""
            ...

        @property
        def detail_clip(self) -> Clip | None:
            """Get/Set the Clip that is currently visible in Lives Detailview."""
            ...

        @detail_clip.setter
        def detail_clip(self, value: Clip | None) -> None: ...

        def detail_clip_has_listener(self, callback: Callable | None, /) -> bool:
            """
            Returns true, if the given listener function or method is connected
            to the property "detail_clip".
            """
            ...

        @property
        def draw_mode(self) -> bool:
            """Get/Set if the Envelope/Note draw mode is enabled."""
            ...

        @draw_mode.setter
        def draw_mode(self, value: bool) -> None: ...

        def draw_mode_has_listener(self, callback: Callable | None, /) -> bool:
            """
            Returns true, if the given listener function or method is connected
            to the property "draw_mode".
            """
            ...

        @property
        def follow_song(self) -> bool:
            """Get/Set if the Arrangerview should scroll to show the playmarker."""
            ...

        @follow_song.setter
        def follow_song(self, value: bool) -> None: ...

        def follow_song_has_listener(self, callback: Callable | None, /) -> bool:
            """
            Returns true, if the given listener function or method is connected
            to the property "follow_song".
            """
            ...

        @property
        def highlighted_clip_slot(self) -> ClipSlot:
            """Get/Set the clip slot, defined via the selected track and scene in the Session.Will be None for Main- and Sendtracks."""
            ...

        @highlighted_clip_slot.setter
        def highlighted_clip_slot(self, value: ClipSlot) -> None: ...

        def remove_detail_clip_listener(self, callback: Callable | None, /) -> None:
            """
            Remove a previously set listener function or method from
            property "detail_clip".
            """
            ...

        def remove_draw_mode_listener(self, callback: Callable | None, /) -> None:
            """
            Remove a previously set listener function or method from
            property "draw_mode".
            """
            ...

        def remove_follow_song_listener(self, callback: Callable | None, /) -> None:
            """
            Remove a previously set listener function or method from
            property "follow_song".
            """
            ...

        def remove_selected_chain_listener(self, callback: Callable | None, /) -> None:
            """
            Remove a previously set listener function or method from
            property "selected_chain".
            """
            ...

        def remove_selected_parameter_listener(self, callback: Callable | None, /) -> None:
            """
            Remove a previously set listener function or method from
            property "selected_parameter".
            """
            ...

        def remove_selected_scene_listener(self, callback: Callable | None, /) -> None:
            """
            Remove a previously set listener function or method from
            property "selected_scene".
            """
            ...

        def remove_selected_track_listener(self, callback: Callable | None, /) -> None:
            """
            Remove a previously set listener function or method from
            property "selected_track".
            """
            ...

        def select_device(self, device: Device | None, should_appoint_device: bool = True, /) -> None:
            """Select the given device."""
            ...

        @property
        def selected_chain(self) -> Chain | None:
            """Get the highlighted chain if available."""
            ...

        @selected_chain.setter
        def selected_chain(self, value: Chain | None) -> None: ...

        def selected_chain_has_listener(self, callback: Callable | None, /) -> bool:
            """
            Returns true, if the given listener function or method is connected
            to the property "selected_chain".
            """
            ...

        @property
        def selected_parameter(self) -> DeviceParameter | None:
            """Get the currently selected device parameter."""
            ...

        def selected_parameter_has_listener(self, callback: Callable | None, /) -> bool:
            """
            Returns true, if the given listener function or method is connected
            to the property "selected_parameter".
            """
            ...

        @property
        def selected_scene(self) -> Scene:
            """Get/Set the current selected scene in Lives Sessionview."""
            ...

        @selected_scene.setter
        def selected_scene(self, value: Scene) -> None: ...

        def selected_scene_has_listener(self, callback: Callable | None, /) -> bool:
            """
            Returns true, if the given listener function or method is connected
            to the property "selected_scene".
            """
            ...

        @property
        def selected_track(self) -> Track:
            """Get/Set the current selected Track in Lives Session or Arrangerview."""
            ...

        @selected_track.setter
        def selected_track(self, value: Track) -> None: ...

        def selected_track_has_listener(self, callback: Callable | None, /) -> bool:
            """
            Returns true, if the given listener function or method is connected
            to the property "selected_track".
            """
            ...

    @property
    def _live_ptr(self) -> int:
        ...

    def add_appointed_device_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "appointed_device" has changed.
        """
        ...

    def add_arrangement_overdub_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "arrangement_overdub" has changed.
        """
        ...

    def add_back_to_arranger_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "back_to_arranger" has changed.
        """
        ...

    def add_can_capture_midi_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "can_capture_midi" has changed.
        """
        ...

    def add_can_jump_to_next_cue_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "can_jump_to_next_cue" has changed.
        """
        ...

    def add_can_jump_to_prev_cue_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "can_jump_to_prev_cue" has changed.
        """
        ...

    def add_clip_trigger_quantization_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "clip_trigger_quantization" has changed.
        """
        ...

    def add_count_in_duration_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "count_in_duration" has changed.
        """
        ...

    def add_cue_points_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "cue_points" has changed.
        """
        ...

    def add_current_song_time_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "current_song_time" has changed.
        """
        ...

    def add_data_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "data" has changed.
        """
        ...

    def add_exclusive_arm_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "exclusive_arm" has changed.
        """
        ...

    def add_groove_amount_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "groove_amount" has changed.
        """
        ...

    def add_is_ableton_link_enabled_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "is_ableton_link_enabled" has changed.
        """
        ...

    def add_is_ableton_link_start_stop_sync_enabled_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "is_ableton_link_start_stop_sync_enabled" has changed.
        """
        ...

    def add_is_counting_in_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "is_counting_in" has changed.
        """
        ...

    def add_is_playing_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "is_playing" has changed.
        """
        ...

    def add_loop_length_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "loop_length" has changed.
        """
        ...

    def add_loop_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "loop" has changed.
        """
        ...

    def add_loop_start_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "loop_start" has changed.
        """
        ...

    def add_metronome_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "metronome" has changed.
        """
        ...

    def add_midi_recording_quantization_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "midi_recording_quantization" has changed.
        """
        ...

    def add_nudge_down_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "nudge_down" has changed.
        """
        ...

    def add_nudge_up_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "nudge_up" has changed.
        """
        ...

    def add_overdub_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "overdub" has changed.
        """
        ...

    def add_punch_in_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "punch_in" has changed.
        """
        ...

    def add_punch_out_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "punch_out" has changed.
        """
        ...

    def add_re_enable_automation_enabled_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "re_enable_automation_enabled" has changed.
        """
        ...

    def add_record_mode_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "record_mode" has changed.
        """
        ...

    def add_return_tracks_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "return_tracks" has changed.
        """
        ...

    def add_root_note_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "root_note" has changed.
        """
        ...

    def add_scale_information_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "scale_information" has changed.
        """
        ...

    def add_scale_intervals_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "scale_intervals" has changed.
        """
        ...

    def add_scale_mode_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "scale_mode" has changed.
        """
        ...

    def add_scale_name_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "scale_name" has changed.
        """
        ...

    def add_scenes_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "scenes" has changed.
        """
        ...

    def add_session_automation_record_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "session_automation_record" has changed.
        """
        ...

    def add_session_record_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "session_record" has changed.
        """
        ...

    def add_session_record_status_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "session_record_status" has changed.
        """
        ...

    def add_signature_denominator_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "signature_denominator" has changed.
        """
        ...

    def add_signature_numerator_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "signature_numerator" has changed.
        """
        ...

    def add_song_length_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "song_length" has changed.
        """
        ...

    def add_start_time_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "start_time" has changed.
        """
        ...

    def add_swing_amount_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "swing_amount" has changed.
        """
        ...

    def add_tempo_follower_enabled_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "tempo_follower_enabled" has changed.
        """
        ...

    def add_tempo_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "tempo" has changed.
        """
        ...

    def add_tracks_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "tracks" has changed.
        """
        ...

    def add_tuning_system_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "tuning_system" has changed.
        """
        ...

    def add_visible_tracks_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "visible_tracks" has changed.
        """
        ...

    @property
    def appointed_device(self) -> Device | None:
        """Read, write, and listen access to the appointed Device"""
        ...

    @appointed_device.setter
    def appointed_device(self, value: Device | None) -> None: ...

    def appointed_device_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "appointed_device".
        """
        ...

    @property
    def arrangement_overdub(self) -> bool:
        """Get/Set the global arrangement overdub state."""
        ...

    @arrangement_overdub.setter
    def arrangement_overdub(self, value: bool) -> None: ...

    def arrangement_overdub_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "arrangement_overdub".
        """
        ...

    @property
    def back_to_arranger(self) -> bool:
        """
        Get/Set if triggering a Clip in the Session, disabled the playback of
        Clips in the Arranger.
        """
        ...

    @back_to_arranger.setter
    def back_to_arranger(self, value: bool) -> None: ...

    def back_to_arranger_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "back_to_arranger".
        """
        ...

    def begin_undo_step(self) -> None:
        ...

    @property
    def can_capture_midi(self) -> bool:
        """Get whether there currently is material to be captured on any tracks."""
        ...

    def can_capture_midi_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "can_capture_midi".
        """
        ...

    @property
    def can_jump_to_next_cue(self) -> bool:
        """
        Returns true when there is a cue marker right to the playing pos that
        we could jump to.
        """
        ...

    def can_jump_to_next_cue_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "can_jump_to_next_cue".
        """
        ...

    @property
    def can_jump_to_prev_cue(self) -> bool:
        """
        Returns true when there is a cue marker left to the playing pos that
        we could jump to.
        """
        ...

    def can_jump_to_prev_cue_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "can_jump_to_prev_cue".
        """
        ...

    @property
    def can_redo(self) -> bool:
        """Returns true if there is an undone action that we can redo."""
        ...

    @property
    def can_undo(self) -> bool:
        """Returns true if there is an action that we can restore."""
        ...

    @property
    def canonical_parent(self) -> None:
        """Get the canonical parent of the song."""
        ...

    def capture_and_insert_scene(self, capture_mode: CaptureMode | int = 0, /) -> None:
        """
        Capture currently playing clips and insert them as a new scene after
        the selected scene. Raises a runtime error if creating a new scene would exceed the limitations.
        """
        ...

    def capture_midi(self, destination: CaptureDestination | int = 0, /) -> None:
        """
        Capture recently played MIDI material from audible tracks.
        If no Destination is given or Destination is set to CaptureDestination.auto, the captured material is inserted into the Session or Arrangement depending on which is visible.
        If Destination is set to CaptureDestination.session or CaptureDestination.arrangement, inserts the material into Session or Arrangement, respectively.
        Raises a limitation error when capturing into the Session and a new scene would have to be created but can't because it would exceed the limitations.
        """
        ...

    @property
    def clip_trigger_quantization(self) -> Quantization:
        """
        Get/Set access to the quantization settings that are used to fire
        Clips in the Session.
        """
        ...

    @clip_trigger_quantization.setter
    def clip_trigger_quantization(self, value: Quantization) -> None: ...

    def clip_trigger_quantization_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "clip_trigger_quantization".
        """
        ...

    def continue_playing(self) -> None:
        """Continue playing the song from the current position"""
        ...

    @property
    def count_in_duration(self) -> int:
        """
        Get the count in duration. Returns an index, mapped as follows:
        0 - None, 1 - 1 Bar, 2 - 2 Bars, 3 - 4 Bars.
        """
        ...

    def count_in_duration_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "count_in_duration".
        """
        ...

    def create_audio_track(self, index: int | None = None, /) -> Track:
        """
        Create a new audio track at the optional given index and return it.If the index is -1,
        the new track is added at the end. It will create a default audio track if possible.
        If the index is invalid or the new track would exceed the limitations, a limitation error is raised.If the index is missing, the track is created after the last selected item
        """
        ...

    def create_midi_track(self, index: int | None = None, /) -> Track:
        """
        Create a new midi track at the optional given index and return it.If the index is -1,
        the new track is added at the end.It will create a default midi track if possible.
        If the index is invalid or the new track would exceed the limitations, a limitation error is raised.If the index is missing, the track is created after the last selected item
        """
        ...

    def create_return_track(self) -> Track:
        """
        Create a new return track at the end and return it. If the new track would exceed
        the limitations, a limitation error is raised.
        If the maximum number of return tracks is exceeded, a RuntimeError is raised.
        """
        ...

    def create_scene(self, index: int | None, /) -> Scene:
        """
        Create a new scene at the given index. If the index is -1,
        the new scene is added at the end. If the index is invalid or
        the new scene would exceed the limitations, a limitation error is raised.
        """
        ...

    @property
    def cue_points(self) -> Vector[CuePoint]:
        """Const access to a list of all cue points of the Live Song."""
        ...

    def cue_points_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "cue_points".
        """
        ...

    @property
    def current_song_time(self) -> float:
        """Get/Set access to the songs current playing position in beats."""
        ...

    @current_song_time.setter
    def current_song_time(self, value: float) -> None: ...

    def current_song_time_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "current_song_time".
        """
        ...

    def data_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "data".
        """
        ...

    def delete_return_track(self, index: int | None, /) -> None:
        """
        Delete the return track with the given index. If no track with this index
        exists, an exception will be raised.
        """
        ...

    def delete_scene(self, index: int | None, /) -> None:
        """
        Delete the scene with the given index. If no scene with this index
        exists, an exception will be raised.
        """
        ...

    def delete_track(self, index: int | None, /) -> None:
        """
        Delete the track with the given index. If no track with this index
        exists, an exception will be raised.
        """
        ...

    def duplicate_scene(self, index: int | None, /) -> None:
        """
        Duplicates a scene and selects the new one.
        Raises a limitation error if creating a new scene would exceed the limitations.
        """
        ...

    def duplicate_track(self, index: int | None, /) -> None:
        """
        Duplicates a track and selects the new one.
        If the track is inside a folded group track, the group track is unfolded.
        Raises a limitation error if creating a new track would exceed the limitations.
        """
        ...

    def end_undo_step(self) -> None:
        ...

    @property
    def exclusive_arm(self) -> bool:
        """Get if Tracks should be armed exclusively by default."""
        ...

    def exclusive_arm_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "exclusive_arm".
        """
        ...

    @property
    def exclusive_solo(self) -> bool:
        """Get if Tracks should be soloed exclusively by default."""
        ...

    @property
    def file_path(self) -> str:
        """Get the current Live Set's path on disk."""
        ...

    def find_device_position(self, device: Device | None, target: Track | Chain | None, target_position: int | None, /) -> int:
        """
        Returns the closest possible position to the given target, where the
        device can be inserted. If inserting is not possible at all (i.e. if
        the device type is wrong), -1 is returned.
        """
        ...

    def force_link_beat_time(self) -> None:
        """
        Force the Link timeline to jump to Lives current beat time.
        Danger: This can cause beat time discontinuities in other connected apps.
        """
        ...

    def get_beats_loop_length(self) -> BeatTime:
        """
        Get const access to the songs loop length, using a
        BeatTime class with the current global set signature.
        """
        ...

    def get_beats_loop_start(self) -> BeatTime:
        """
        Get const access to the songs loop start, using a
        BeatTime class with the current global set signature.
        """
        ...

    def get_current_beats_song_time(self) -> BeatTime:
        """
        Get const access to the songs current playing position, using a
        BeatTime class with the current global set signature.
        """
        ...

    def get_current_smpte_song_time(self, smpte_format: int | None, /) -> SmptTime:
        """
        Get const access to the songs current playing position, by specifying
        the SMPTE format in which you would like to receive the time.
        """
        ...

    def get_data(self, key: str | None, default_value: Any | None, /) -> Any:
        """Get data for the given key, that was previously stored using set_data."""
        ...

    @property
    def groove_amount(self) -> float:
        """
        Get/Set the global groove amount, that adjust all setup grooves
        in all clips.
        """
        ...

    @groove_amount.setter
    def groove_amount(self, value: float) -> None: ...

    def groove_amount_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "groove_amount".
        """
        ...

    @property
    def groove_pool(self) -> GroovePool:
        """Get the groove pool."""
        ...

    @property
    def is_ableton_link_enabled(self) -> bool:
        """Enable/disable Ableton Link."""
        ...

    @is_ableton_link_enabled.setter
    def is_ableton_link_enabled(self, value: bool) -> None: ...

    def is_ableton_link_enabled_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "is_ableton_link_enabled".
        """
        ...

    @property
    def is_ableton_link_start_stop_sync_enabled(self) -> bool:
        """Enable/disable Ableton Link Start Stop Sync."""
        ...

    @is_ableton_link_start_stop_sync_enabled.setter
    def is_ableton_link_start_stop_sync_enabled(self, value: bool) -> None: ...

    def is_ableton_link_start_stop_sync_enabled_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "is_ableton_link_start_stop_sync_enabled".
        """
        ...

    @property
    def is_counting_in(self) -> bool:
        """Get whether currently counting in."""
        ...

    def is_counting_in_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "is_counting_in".
        """
        ...

    def is_cue_point_selected(self) -> bool:
        """Return true if the global playing pos is currently on a cue point."""
        ...

    @property
    def is_playing(self) -> bool:
        """Returns true if the Song is currently playing."""
        ...

    @is_playing.setter
    def is_playing(self, value: bool) -> None: ...

    def is_playing_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "is_playing".
        """
        ...

    def jump_by(self, beats: float | None, /) -> None:
        """Set a new playing pos, relative to the current one."""
        ...

    def jump_to_next_cue(self) -> None:
        """Jump to the next cue (marker) if possible."""
        ...

    def jump_to_prev_cue(self) -> None:
        """Jump to the prior cue (marker) if possible."""
        ...

    @property
    def last_event_time(self) -> float:
        """
        Return the time of the last set event in the song. In contrary to
        song_length, this will not add some extra beats that are mostly needed
        for Display purposes in the Arrangerview.
        """
        ...

    @property
    def loop(self) -> bool:
        """
        Get/Set the looping flag that en/disables the usage of the global
        loop markers in the song.
        """
        ...

    @loop.setter
    def loop(self, value: bool) -> None: ...

    def loop_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "loop".
        """
        ...

    @property
    def loop_length(self) -> float:
        """Get/Set the length of the global loop marker position in beats."""
        ...

    @loop_length.setter
    def loop_length(self, value: float) -> None: ...

    def loop_length_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "loop_length".
        """
        ...

    @property
    def loop_start(self) -> float:
        """Get/Set the start of the global loop marker position in beats."""
        ...

    @loop_start.setter
    def loop_start(self, value: float) -> None: ...

    def loop_start_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "loop_start".
        """
        ...

    @property
    def master_track(self) -> Track:
        """Access to the Main Track (always available)"""
        ...

    @property
    def metronome(self) -> bool:
        """Get/Set if the metronom is audible."""
        ...

    @metronome.setter
    def metronome(self, value: bool) -> None: ...

    def metronome_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "metronome".
        """
        ...

    @property
    def midi_recording_quantization(self) -> RecordingQuantization:
        """
        Get/Set access to the settings that are used to quantize
        MIDI recordings.
        """
        ...

    @midi_recording_quantization.setter
    def midi_recording_quantization(self, value: RecordingQuantization) -> None: ...

    def midi_recording_quantization_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "midi_recording_quantization".
        """
        ...

    def move_device(self, device: Device | None, target: Track | Chain | None, target_position: int | None, /) -> int:
        """Move a device into the target at the given position, where 0 moves it before the first device and len(devices) moves it to the end of the device chain.If the device cannot be moved to this position, the nearest possible position is chosen. If the device type is not valid, a runtime error is raised.Returns the index, where the device was moved to."""
        ...

    @property
    def name(self) -> str:
        """Get the current Live Set's name."""
        ...

    @property
    def nudge_down(self) -> bool:
        """Get/Set the status of the nudge down button."""
        ...

    @nudge_down.setter
    def nudge_down(self, value: bool) -> None: ...

    def nudge_down_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "nudge_down".
        """
        ...

    @property
    def nudge_up(self) -> bool:
        """Get/Set the status of the nudge up button."""
        ...

    @nudge_up.setter
    def nudge_up(self, value: bool) -> None: ...

    def nudge_up_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "nudge_up".
        """
        ...

    @property
    def overdub(self) -> bool:
        """
        Legacy hook for Live 8 overdub state. Now hooks to
        session record, but never starts playback.
        """
        ...

    @overdub.setter
    def overdub(self, value: bool) -> None: ...

    def overdub_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "overdub".
        """
        ...

    def play_selection(self) -> None:
        """
        Start playing the current set selection, or do nothing if
        no selection is set.
        """
        ...

    @property
    def punch_in(self) -> bool:
        """
        Get/Set the flag that will enable recording as soon as the Song plays
        and hits the global loop start region.
        """
        ...

    @punch_in.setter
    def punch_in(self, value: bool) -> None: ...

    def punch_in_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "punch_in".
        """
        ...

    @property
    def punch_out(self) -> bool:
        """
        Get/Set the flag that will disable recording as soon as the Song plays
        and hits the global loop end region.
        """
        ...

    @punch_out.setter
    def punch_out(self, value: bool) -> None: ...

    def punch_out_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "punch_out".
        """
        ...

    def re_enable_automation(self) -> None:
        """Discards overrides of automated parameters."""
        ...

    @property
    def re_enable_automation_enabled(self) -> bool:
        """Returns true if some automated parameter has been overriden"""
        ...

    def re_enable_automation_enabled_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "re_enable_automation_enabled".
        """
        ...

    @property
    def record_mode(self) -> bool:
        """Get/Set the state of the global recording flag."""
        ...

    @record_mode.setter
    def record_mode(self, value: bool) -> None: ...

    def record_mode_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "record_mode".
        """
        ...

    def redo(self) -> str:
        """Redo the last action that was undone."""
        ...

    def remove_appointed_device_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "appointed_device".
        """
        ...

    def remove_arrangement_overdub_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "arrangement_overdub".
        """
        ...

    def remove_back_to_arranger_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "back_to_arranger".
        """
        ...

    def remove_can_capture_midi_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "can_capture_midi".
        """
        ...

    def remove_can_jump_to_next_cue_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "can_jump_to_next_cue".
        """
        ...

    def remove_can_jump_to_prev_cue_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "can_jump_to_prev_cue".
        """
        ...

    def remove_clip_trigger_quantization_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "clip_trigger_quantization".
        """
        ...

    def remove_count_in_duration_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "count_in_duration".
        """
        ...

    def remove_cue_points_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "cue_points".
        """
        ...

    def remove_current_song_time_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "current_song_time".
        """
        ...

    def remove_data_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "data".
        """
        ...

    def remove_exclusive_arm_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "exclusive_arm".
        """
        ...

    def remove_groove_amount_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "groove_amount".
        """
        ...

    def remove_is_ableton_link_enabled_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "is_ableton_link_enabled".
        """
        ...

    def remove_is_ableton_link_start_stop_sync_enabled_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "is_ableton_link_start_stop_sync_enabled".
        """
        ...

    def remove_is_counting_in_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "is_counting_in".
        """
        ...

    def remove_is_playing_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "is_playing".
        """
        ...

    def remove_loop_length_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "loop_length".
        """
        ...

    def remove_loop_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "loop".
        """
        ...

    def remove_loop_start_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "loop_start".
        """
        ...

    def remove_metronome_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "metronome".
        """
        ...

    def remove_midi_recording_quantization_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "midi_recording_quantization".
        """
        ...

    def remove_nudge_down_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "nudge_down".
        """
        ...

    def remove_nudge_up_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "nudge_up".
        """
        ...

    def remove_overdub_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "overdub".
        """
        ...

    def remove_punch_in_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "punch_in".
        """
        ...

    def remove_punch_out_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "punch_out".
        """
        ...

    def remove_re_enable_automation_enabled_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "re_enable_automation_enabled".
        """
        ...

    def remove_record_mode_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "record_mode".
        """
        ...

    def remove_return_tracks_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "return_tracks".
        """
        ...

    def remove_root_note_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "root_note".
        """
        ...

    def remove_scale_information_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "scale_information".
        """
        ...

    def remove_scale_intervals_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "scale_intervals".
        """
        ...

    def remove_scale_mode_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "scale_mode".
        """
        ...

    def remove_scale_name_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "scale_name".
        """
        ...

    def remove_scenes_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "scenes".
        """
        ...

    def remove_session_automation_record_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "session_automation_record".
        """
        ...

    def remove_session_record_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "session_record".
        """
        ...

    def remove_session_record_status_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "session_record_status".
        """
        ...

    def remove_signature_denominator_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "signature_denominator".
        """
        ...

    def remove_signature_numerator_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "signature_numerator".
        """
        ...

    def remove_song_length_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "song_length".
        """
        ...

    def remove_start_time_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "start_time".
        """
        ...

    def remove_swing_amount_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "swing_amount".
        """
        ...

    def remove_tempo_follower_enabled_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "tempo_follower_enabled".
        """
        ...

    def remove_tempo_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "tempo".
        """
        ...

    def remove_tracks_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "tracks".
        """
        ...

    def remove_tuning_system_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "tuning_system".
        """
        ...

    def remove_visible_tracks_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "visible_tracks".
        """
        ...

    @property
    def return_tracks(self) -> Vector[Track]:
        """Const access to the list of available Return Tracks."""
        ...

    def return_tracks_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "return_tracks".
        """
        ...

    @property
    def root_note(self) -> int:
        """Set and access the root (i.e. key) of the song. The root can be a number between 0 and 11, with 0 corresponding to C and 11 corresponding to B."""
        ...

    @root_note.setter
    def root_note(self, value: int) -> None: ...

    def root_note_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "root_note".
        """
        ...

    def scale_information_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "scale_information".
        """
        ...

    @property
    def scale_intervals(self) -> IntVector:
        """Reports the current scale's intervals as a list of integers, starting with the root and representing the number of halfsteps (e.g. Major -> 0, 2, 4, 5, 7, 9, 11)"""
        ...

    def scale_intervals_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "scale_intervals".
        """
        ...

    @property
    def scale_mode(self) -> bool:
        """Access to the Scale Mode setting in Live. When on, key tracks that belong to the currently selected scale are highlighted in Live's MIDI Note Editor, and pitch-based parameters in MIDI Tools and Devices can be edited in scale degrees rather than semitones."""
        ...

    @scale_mode.setter
    def scale_mode(self, value: bool) -> None: ...

    def scale_mode_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "scale_mode".
        """
        ...

    @property
    def scale_name(self) -> str:
        """
        Set and access the currently selected scale by name. The default scale names that can be saved with a set and recalled are
        'Major', 'Minor', 'Dorian', 'Mixolydian' ,'Lydian' ,'Phrygian' ,'Locrian',
        'Whole Tone', 'Half-whole Dim.', 'Whole-half Dim.', 'Minor Blues',
        'Minor Pentatonic', 'Major Pentatonic', 'Harmonic Minor', 'Harmonic Major',
        'Dorian #4', 'Phrygian Dominant', 'Melodic Minor', 'Lydian Augmented',
        'Lydian Dominant', 'Super Locrian', 'Bhairav', 'Hungarian Minor',
        '8-Tone Spanish', 'Hirajoshi', 'In-Sen', 'Iwato', 'Kumoi', 'Pelog Selisir',
        'Pelog Tembung', 'Messiaen 3', 'Messiaen 4', 'Messiaen 5', 'Messiaen 6',
        'Messiaen 7'
        """
        ...

    @scale_name.setter
    def scale_name(self, value: str) -> None: ...

    def scale_name_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "scale_name".
        """
        ...

    @property
    def scenes(self) -> Vector[Scene]:
        """Const access to a list of all Scenes in the Live Song."""
        ...

    def scenes_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "scenes".
        """
        ...

    def scrub_by(self, beats: float | None, /) -> None:
        """Same as jump_by, but does not stop playback."""
        ...

    @property
    def select_on_launch(self) -> bool:
        """Get if Scenes and Clips should be selected when fired."""
        ...

    @property
    def session_automation_record(self) -> bool:
        """Returns true if automation recording is enabled."""
        ...

    @session_automation_record.setter
    def session_automation_record(self, value: bool) -> None: ...

    def session_automation_record_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "session_automation_record".
        """
        ...

    @property
    def session_record(self) -> bool:
        """Get/Set the session record state."""
        ...

    @session_record.setter
    def session_record(self, value: bool) -> None: ...

    def session_record_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "session_record".
        """
        ...

    @property
    def session_record_status(self) -> int:
        """Get the session slot-recording state."""
        ...

    def session_record_status_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "session_record_status".
        """
        ...

    def set_data(self, key: str | None, value: Any | None, /) -> None:
        """Store data for the given key in this object. The data is persistent and will be restored when loading the Live Set."""
        ...

    def set_or_delete_cue(self) -> None:
        """
        When a cue is selected, it gets deleted. If no cue is selected,
        a new cue is created at the current global songtime.
        """
        ...

    @property
    def signature_denominator(self) -> int:
        """Get/Set access to the global signature denominator of the Song."""
        ...

    @signature_denominator.setter
    def signature_denominator(self, value: int) -> None: ...

    def signature_denominator_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "signature_denominator".
        """
        ...

    @property
    def signature_numerator(self) -> int:
        """Get/Set access to the global signature numerator of the Song."""
        ...

    @signature_numerator.setter
    def signature_numerator(self, value: int) -> None: ...

    def signature_numerator_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "signature_numerator".
        """
        ...

    @property
    def song_length(self) -> float:
        """
        Return the time of the last set event in the song, plus som extra beats
        that are usually added for better navigation in the arrangerview.
        """
        ...

    def song_length_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "song_length".
        """
        ...

    def start_playing(self) -> None:
        """Start playing from the startmarker"""
        ...

    @property
    def start_time(self) -> float:
        """
        Get/Set access to the songs current start time in beats. The set time
        may be overridden by the current loop/locator start time.
        """
        ...

    @start_time.setter
    def start_time(self, value: float) -> None: ...

    def start_time_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "start_time".
        """
        ...

    def stop_all_clips(self, quantized: bool = True, /) -> None:
        """Stop all playing Clips (if any) but continue playing the Song."""
        ...

    def stop_playing(self) -> None:
        """Stop playing the Song."""
        ...

    @property
    def swing_amount(self) -> float:
        """Get/Set access to the amount of swing that is applied when adding or quantizing notes to MIDI clips"""
        ...

    @swing_amount.setter
    def swing_amount(self, value: float) -> None: ...

    def swing_amount_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "swing_amount".
        """
        ...

    def tap_tempo(self) -> None:
        """Trigger the tap tempo function."""
        ...

    @property
    def tempo(self) -> float:
        """Get/Set the global project tempo."""
        ...

    @tempo.setter
    def tempo(self, value: float) -> None: ...

    @property
    def tempo_follower_enabled(self) -> bool:
        """Get/Set whether the Tempo Follower is controlling the tempo. The Tempo Follower Toggle must be made visible in the preferences for this property to be effective."""
        ...

    @tempo_follower_enabled.setter
    def tempo_follower_enabled(self, value: bool) -> None: ...

    def tempo_follower_enabled_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "tempo_follower_enabled".
        """
        ...

    def tempo_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "tempo".
        """
        ...

    @property
    def tracks(self) -> Vector[Track]:
        """
        Const access to a list of all Player Tracks in the Live Song, excluding
        the return and Main Track (see also Song.send_tracks and Song.master_track).
        At least one MIDI or Audio Track is always available.
        """
        ...

    def tracks_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "tracks".
        """
        ...

    def trigger_session_record(self, record_length: float = 1.7976931348623157e+308, /) -> None:
        """Triggers a new session recording."""
        ...

    @property
    def tuning_system(self) -> TuningSystem:
        """Access the currently active tuning system."""
        ...

    def tuning_system_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "tuning_system".
        """
        ...

    def undo(self) -> str:
        """Undo the last action that was made."""
        ...

    @property
    def view(self) -> View:
        """
        Representing the view aspects of a Live document:
        The Session and Arrangerview.
        """
        ...

    @property
    def visible_tracks(self) -> Vector[Track]:
        """
        Const access to a list of all visible Player Tracks in the Live Song, excluding
        the return and Main Track (see also Song.send_tracks and Song.master_track).
        At least one MIDI or Audio Track is always available.
        """
        ...

    def visible_tracks_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "visible_tracks".
        """
        ...

class BeatTime:
    """Represents a Time, splitted into Bars, Beats, SubDivision and Ticks."""

    def __init__(self) -> None: ...

    @property
    def bars(self) -> int:
        ...

    @bars.setter
    def bars(self, value: int) -> None: ...

    @property
    def beats(self) -> int:
        ...

    @beats.setter
    def beats(self, value: int) -> None: ...

    @property
    def sub_division(self) -> int:
        ...

    @sub_division.setter
    def sub_division(self, value: int) -> None: ...

    @property
    def ticks(self) -> int:
        ...

    @ticks.setter
    def ticks(self, value: int) -> None: ...

class CaptureDestination(int):
    """The destination for MIDI capture."""
    auto: int = 0
    session: int = 1
    arrangement: int = 2

class CaptureMode(int):
    """The capture mode that is used for capture and insert scene."""
    all: int = 0
    all_except_selected: int = 1

class CuePoint(LomObject):
    """Represents a 'Marker' in the arrangement."""

    @property
    def _live_ptr(self) -> int:
        ...

    def add_name_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "name" has changed.
        """
        ...

    def add_time_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "time" has changed.
        """
        ...

    @property
    def canonical_parent(self) -> Song:
        """Get the canonical parent of the cue point."""
        ...

    def jump(self) -> None:
        """
        When the Song is playing, set the playing-position quantized to
        this Cuepoint's time. When not playing, simply move the start
        playing position.
        """
        ...

    @property
    def name(self) -> str:
        """Get/Set/Listen to the name of this CuePoint, as visible in the arranger."""
        ...

    @name.setter
    def name(self, value: str) -> None: ...

    def name_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "name".
        """
        ...

    def remove_name_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "name".
        """
        ...

    def remove_time_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "time".
        """
        ...

    @property
    def time(self) -> float:
        """Get/Listen to the CuePoint's time in beats."""
        ...

    def time_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "time".
        """
        ...

class Quantization(int):
    q_no_q: int = 0
    q_8_bars: int = 1
    q_4_bars: int = 2
    q_2_bars: int = 3
    q_bar: int = 4
    q_half: int = 5
    q_half_triplet: int = 6
    q_quarter: int = 7
    q_quarter_triplet: int = 8
    q_eight: int = 9
    q_eight_triplet: int = 10
    q_sixtenth: int = 11
    q_sixtenth_triplet: int = 12
    q_thirtytwoth: int = 13

class RecordingQuantization(int):
    rec_q_no_q: int = 0
    rec_q_quarter: int = 1
    rec_q_eight: int = 2
    rec_q_eight_triplet: int = 3
    rec_q_eight_eight_triplet: int = 4
    rec_q_sixtenth: int = 5
    rec_q_sixtenth_triplet: int = 6
    rec_q_sixtenth_sixtenth_triplet: int = 7
    rec_q_thirtysecond: int = 8

class SessionRecordStatus(int):
    off: int = 0
    on: int = 1
    transition: int = 2

class SmptTime:
    """
    Represents a Time, split into Hours, Minutes, Seconds and Frames.
    The frame type must be specified when calling a function that returns
    a SmptTime.
    """

    def __init__(self) -> None: ...

    @property
    def frames(self) -> int:
        ...

    @frames.setter
    def frames(self, value: int) -> None: ...

    @property
    def hours(self) -> int:
        ...

    @hours.setter
    def hours(self, value: int) -> None: ...

    @property
    def minutes(self) -> int:
        ...

    @minutes.setter
    def minutes(self, value: int) -> None: ...

    @property
    def seconds(self) -> int:
        ...

    @seconds.setter
    def seconds(self, value: int) -> None: ...

class TimeFormat(int):
    ms_time: int = 0
    smpte_24: int = 1
    smpte_25: int = 2
    smpte_30: int = 3
    smpte_30_drop: int = 4
    smpte_29: int = 5

def get_all_scales_ordered() -> tuple[tuple[str, tuple[int, ...]], ...]:
    """Get an ordered tuple of tuples of all available scale names to intervals."""
    ...

__all__ = ['Song', 'BeatTime', 'CaptureDestination', 'CaptureMode', 'CuePoint', 'Quantization', 'RecordingQuantization', 'SessionRecordStatus', 'SmptTime', 'TimeFormat', 'get_all_scales_ordered']



from __future__ import annotations
from typing import TYPE_CHECKING, Any, Callable, Iterable

if TYPE_CHECKING:
    from Live.Base import Vector
    from Live.ClipSlot import ClipSlot
    from Live.LomObject import LomObject
    from Live.Song import Song



class Scene(LomObject):
    """This class represents an series of ClipSlots in Lives Sessionview matrix."""

    @property
    def _live_ptr(self) -> int:
        ...

    def add_clip_slots_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "clip_slots" has changed.
        """
        ...

    def add_color_index_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "color_index" has changed.
        """
        ...

    def add_color_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "color" has changed.
        """
        ...

    def add_is_triggered_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "is_triggered" has changed.
        """
        ...

    def add_name_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "name" has changed.
        """
        ...

    def add_tempo_enabled_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "tempo_enabled" has changed.
        """
        ...

    def add_tempo_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "tempo" has changed.
        """
        ...

    def add_time_signature_denominator_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "time_signature_denominator" has changed.
        """
        ...

    def add_time_signature_enabled_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "time_signature_enabled" has changed.
        """
        ...

    def add_time_signature_numerator_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "time_signature_numerator" has changed.
        """
        ...

    @property
    def canonical_parent(self) -> Song:
        """Get the canonical parent of the scene."""
        ...

    @property
    def clip_slots(self) -> Vector[ClipSlot]:
        """return a list of clipslots (see class AClipSlot) that this scene covers."""
        ...

    def clip_slots_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "clip_slots".
        """
        ...

    @property
    def color(self) -> int:
        """Get/set access to the color of the scene (RGB)."""
        ...

    @color.setter
    def color(self, value: int) -> None: ...

    def color_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "color".
        """
        ...

    @property
    def color_index(self) -> int | None:
        """Get/set access to the color index of the scene. Can be None for no color."""
        ...

    @color_index.setter
    def color_index(self, value: int | None) -> None: ...

    def color_index_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "color_index".
        """
        ...

    def fire(self, force_legato: bool = False, can_select_scene_on_launch: bool = True, /) -> None:
        """
        Fire the scene directly. Will fire all clipslots that this scene owns and
        select the scene itself.
        """
        ...

    def fire_as_selected(self, force_legato: bool = False, /) -> None:
        """
        Fire the selected scene. Will fire all clipslots that this scene owns and
        select the next scene if necessary.
        """
        ...

    @property
    def is_empty(self) -> bool:
        """Returns True if all clip slots of this scene are empty."""
        ...

    @property
    def is_triggered(self) -> bool:
        """Const access to the scene's trigger state."""
        ...

    def is_triggered_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "is_triggered".
        """
        ...

    @property
    def name(self) -> str:
        """Get/Set the name of the scene."""
        ...

    @name.setter
    def name(self, value: str) -> None: ...

    def name_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "name".
        """
        ...

    def remove_clip_slots_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "clip_slots".
        """
        ...

    def remove_color_index_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "color_index".
        """
        ...

    def remove_color_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "color".
        """
        ...

    def remove_is_triggered_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "is_triggered".
        """
        ...

    def remove_name_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "name".
        """
        ...

    def remove_tempo_enabled_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "tempo_enabled".
        """
        ...

    def remove_tempo_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "tempo".
        """
        ...

    def remove_time_signature_denominator_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "time_signature_denominator".
        """
        ...

    def remove_time_signature_enabled_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "time_signature_enabled".
        """
        ...

    def remove_time_signature_numerator_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "time_signature_numerator".
        """
        ...

    def set_fire_button_state(self, state: bool | None, /) -> None:
        """Set the scene's fire button state directly. Supports all launch modes."""
        ...

    @property
    def tempo(self) -> float:
        """
        Get/Set the tempo value of the scene.
        The song will use the scene's tempo as soon as the scene is fired.
        Returns -1 if the scene has no tempo property.
        """
        ...

    @tempo.setter
    def tempo(self, value: float) -> None: ...

    @property
    def tempo_enabled(self) -> bool:
        """
        Get/Set the active state of the scene tempo.
        When disabled, the scene will use the song's tempo,and the tempo value returned will be -1Returns a bool indicating the state of the scene's tempo
        """
        ...

    @tempo_enabled.setter
    def tempo_enabled(self, value: bool) -> None: ...

    def tempo_enabled_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "tempo_enabled".
        """
        ...

    def tempo_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "tempo".
        """
        ...

    @property
    def time_signature_denominator(self) -> int:
        """
        Get/Set the scene's time signature denominator.
        The song will use the scene's time signature as soon as the scene is fired.
        Returns -1 if the scene has no time signature property.
        """
        ...

    @time_signature_denominator.setter
    def time_signature_denominator(self, value: int) -> None: ...

    def time_signature_denominator_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "time_signature_denominator".
        """
        ...

    @property
    def time_signature_enabled(self) -> bool:
        """
        Get the active state of the scene time signature.
        When disabled, the scene will use the song's time signature,and the time signature values returned will be -1Returns a bool indicating the state of the scene's time signature
        """
        ...

    @time_signature_enabled.setter
    def time_signature_enabled(self, value: bool) -> None: ...

    def time_signature_enabled_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "time_signature_enabled".
        """
        ...

    @property
    def time_signature_numerator(self) -> int:
        """
        Get/Set the scene's time signature numerator.
        The song will use the scene's time signature as soon as the scene is fired.
        Returns -1 if the scene has no time signature property.
        """
        ...

    @time_signature_numerator.setter
    def time_signature_numerator(self, value: int) -> None: ...

    def time_signature_numerator_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "time_signature_numerator".
        """
        ...

__all__ = ['Scene']



from __future__ import annotations
from typing import TYPE_CHECKING, Any, Callable, Iterable

if TYPE_CHECKING:
    from Live.Base import Vector
    from Live.Chain import Chain
    from Live.Device import Device
    from Live.DeviceParameter import DeviceParameter
    from Live.LomObject import LomObject



class ChainMixerDevice(LomObject):
    """
    This class represents a Chain's Mixer Device in Live, which gives you
    access to the Volume, Panning, and Send properties of a Chain.
    """

    @property
    def _live_ptr(self) -> int:
        ...

    def add_sends_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "sends" has changed.
        """
        ...

    @property
    def canonical_parent(self) -> Chain:
        """Get the canonical parent of the mixer device."""
        ...

    @property
    def chain_activator(self) -> DeviceParameter:
        """Const access to the Chain's Activator Device Parameter."""
        ...

    @property
    def panning(self) -> DeviceParameter:
        """Const access to the Chain's Panning Device Parameter."""
        ...

    def remove_sends_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "sends".
        """
        ...

    @property
    def sends(self) -> Vector:
        """Const access to the Chain's list of Send Amount Device Parameters."""
        ...

    def sends_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "sends".
        """
        ...

    @property
    def volume(self) -> DeviceParameter:
        """Const access to the Chain's Volume Device Parameter."""
        ...

__all__ = ['ChainMixerDevice']



from __future__ import annotations
from typing import TYPE_CHECKING, Any, Callable, Generic, Iterable, Iterator, TypeVar, overload

T = TypeVar('T')

if TYPE_CHECKING:
    from Live.Base import Vector



class ListenerHandle:
    """This class represents a Python listener when connected to a Live property."""

    def disconnect(self) -> None:
        """Disconnects the listener from its property"""
        ...

    @property
    def listener_func(self) -> Callable:
        """Returns the original function"""
        ...

    @property
    def listener_self(self) -> Any:
        """Returns the weak reference to original self, if it was a bound method"""
        ...

    @property
    def name(self) -> str:
        """Prints the name of the property that this listener is connected to"""
        ...

class ListenerVector(Vector[ListenerHandle]):
    """A read only container for accessing a list of listeners."""

    def append(self, value: ListenerHandle | None, /) -> None:
        ...

    def extend(self, values: Iterable[ListenerHandle] | None, /) -> None:
        ...

__all__ = ['ListenerHandle', 'ListenerVector']



from __future__ import annotations
from typing import TYPE_CHECKING, Any, Callable, Iterable

if TYPE_CHECKING:
    from Live.Base import StringVector
    from Live.Device import ATimeableValueVector, Device, DeviceType
    from Live.Track import Track



class HybridReverbDevice(Device):
    """This class represents a Hybrid Reverb device."""

    @property
    def _live_ptr(self) -> int:
        ...

    def add_ir_attack_time_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "ir_attack_time" has changed.
        """
        ...

    def add_ir_category_index_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "ir_category_index" has changed.
        """
        ...

    def add_ir_decay_time_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "ir_decay_time" has changed.
        """
        ...

    def add_ir_file_index_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "ir_file_index" has changed.
        """
        ...

    def add_ir_file_list_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "ir_file_list" has changed.
        """
        ...

    def add_ir_size_factor_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "ir_size_factor" has changed.
        """
        ...

    def add_ir_time_shaping_on_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "ir_time_shaping_on" has changed.
        """
        ...

    @property
    def can_compare_ab(self) -> bool:
        """Returns true if the Device has the capability to AB compare."""
        ...

    @property
    def can_have_chains(self) -> bool:
        """Returns true if the device is a rack."""
        ...

    @property
    def can_have_drum_pads(self) -> bool:
        """Returns true if the device is a drum rack."""
        ...

    @property
    def canonical_parent(self) -> Track:
        """Get the canonical parent of the Device."""
        ...

    @property
    def class_display_name(self) -> str:
        """Return const access to the name of the device's class name as displayed in Live's browser and device chain"""
        ...

    @property
    def class_name(self) -> str:
        """Return const access to the name of the device's class."""
        ...

    @property
    def ir_attack_time(self) -> float:
        """Return the current IrAttackTime"""
        ...

    @ir_attack_time.setter
    def ir_attack_time(self, value: float) -> None: ...

    def ir_attack_time_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "ir_attack_time".
        """
        ...

    @property
    def ir_category_index(self) -> int:
        """Return the current IR category index"""
        ...

    @ir_category_index.setter
    def ir_category_index(self, value: int) -> None: ...

    def ir_category_index_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "ir_category_index".
        """
        ...

    @property
    def ir_category_list(self) -> StringVector:
        """Return the current IR categories list"""
        ...

    @property
    def ir_decay_time(self) -> float:
        """Return the current IrDecayTime"""
        ...

    @ir_decay_time.setter
    def ir_decay_time(self, value: float) -> None: ...

    def ir_decay_time_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "ir_decay_time".
        """
        ...

    @property
    def ir_file_index(self) -> int:
        """Return the current IR file index"""
        ...

    @ir_file_index.setter
    def ir_file_index(self, value: int) -> None: ...

    def ir_file_index_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "ir_file_index".
        """
        ...

    @property
    def ir_file_list(self) -> StringVector:
        """Return the current IR file list"""
        ...

    def ir_file_list_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "ir_file_list".
        """
        ...

    @property
    def ir_size_factor(self) -> float:
        """Return the current IrSizeFactor"""
        ...

    @ir_size_factor.setter
    def ir_size_factor(self, value: float) -> None: ...

    def ir_size_factor_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "ir_size_factor".
        """
        ...

    @property
    def ir_time_shaping_on(self) -> bool:
        """Return the current IrTimeShapingOn"""
        ...

    @ir_time_shaping_on.setter
    def ir_time_shaping_on(self, value: bool) -> None: ...

    def ir_time_shaping_on_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "ir_time_shaping_on".
        """
        ...

    @property
    def is_active(self) -> bool:
        """Return const access to whether this device is active. This will be false bothwhen the device is off and when it's inside a rack device which is off."""
        ...

    @property
    def is_using_compare_preset_b(self) -> bool:
        """Returns whether the Device has loaded the preset in compare slot B. Only relevant if can_compare_ab, otherwise errors."""
        ...

    @is_using_compare_preset_b.setter
    def is_using_compare_preset_b(self, value: bool) -> None: ...

    @property
    def latency_in_ms(self) -> float:
        """Returns the latency of the device in ms."""
        ...

    @property
    def latency_in_samples(self) -> int:
        """Returns the latency of the device in samples."""
        ...

    @property
    def name(self) -> str:
        """Return access to the name of the device."""
        ...

    @name.setter
    def name(self, value: str) -> None: ...

    @property
    def parameters(self) -> ATimeableValueVector:
        """Const access to the list of available automatable parameters for this device."""
        ...

    def remove_ir_attack_time_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "ir_attack_time".
        """
        ...

    def remove_ir_category_index_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "ir_category_index".
        """
        ...

    def remove_ir_decay_time_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "ir_decay_time".
        """
        ...

    def remove_ir_file_index_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "ir_file_index".
        """
        ...

    def remove_ir_file_list_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "ir_file_list".
        """
        ...

    def remove_ir_size_factor_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "ir_size_factor".
        """
        ...

    def remove_ir_time_shaping_on_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "ir_time_shaping_on".
        """
        ...

    @property
    def type(self) -> DeviceType:
        """Return the type of the device."""
        ...

    @property
    def view(self) -> Device.View:
        """Representing the view aspects of a device."""
        ...

__all__ = ['HybridReverbDevice']
